window.Gleam.initSearch([{"doc":"gleatter","title":"gleatter","content":"# gleatter\n\n[![Package Version](https://img.shields.io/hexpm/v/gleatter)](https://hex.pm/packages/gleatter)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gleatter/)\n\n**Typed route definitions for seamless fullstack development !**\n\n## Installation\n\n```sh\ngleam add gleatter\n```\n\n## Usage\n\n```gleam\nimport gleatter\nimport gleatter/body\nimport gleatter/route\nimport gleatter/path\nimport gleatter/lustre as gleatter_lustre\nimport gleam/json\nimport gleam/dynamic\nimport gleam/http\n\npub type Todo {\n  Todo(id: String, title: String)\n}\n\nfn encoder(t: Todo) -&gt; json.Json {\n  json.object([\n    #(&quot;id&quot;, json.string(t.id)),\n    #(&quot;title&quot;, json.string(t.title)),\n  ])\n}\n\nfn decoder(\n  value: dynamic.Dynamic,\n) -&gt; Result(Todo, List(dynamic.DecodeError)) {\n  value\n  |&gt; dynamic.decode2(\n    Todo,\n    dynamic.field(&quot;id&quot;, dynamic.string),\n    dynamic.field(&quot;title&quot;, dynamic.string),\n  )\n}\n\npub fn main() {\n  let get_all_todos_route = route.new()\n    |&gt; with_path(path.static_path([&quot;todos&quot;]))\n    |&gt; with_response_body(body.json_body(encoder, decoder))\n\n  // This route can be shared and used by both frontend and backend applications\n\n  // Here is an example for a frontend lustre application using gleatter_lustre\n  gleatter_lustre.create_factory()\n    |&gt; gleatter_lustre.with_port(2345)\n    |&gt; gleatter_lustre.for_route(get_all_todos_route)\n    |&gt; gleatter_lustre.with_path(Nil) // For now, this is necessary\n    |&gt; gleatter_lustre.send(\n      fn(todos) { ServerSentTodos(todos) }, // ServerSentTodos is a Lustre message\n      fn(_err) { effect.none() } // This is for error handling\n    )\n}\n```\n\nSee also the `gleatter/service` module for easy route definitions.\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gleatter&gt;.\n\n## Features\n\n- Route definitions with full type support\n- Erlang and Javascript targets\n- Easy service route definitions\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"gleatter","title":"GleatterError","content":"pub type GleatterError {\n  RouteError(msg: String)\n  JsonDecodeError(err: json.DecodeError)\n}\n Errors that can occur in Gleatter code\nRouteError(msg: String)\n\n\nJsonDecodeError(err: json.DecodeError)\n\n\nSynonyms:\ngleatter.GleatterError\ngleatter GleatterError","url":"gleatter.html#GleatterError"},{"doc":"gleatter","title":"gleatter","content":"","url":"gleatter.html"},{"doc":"gleatter/body","title":"BodyConverter","content":"pub type BodyConverter(body_type) {\n  BodyConverter(\n    encoder: fn(body_type) -&gt; string_builder.StringBuilder,\n    decoder: fn(String) -&gt;\n      Result(body_type, gleatter.GleatterError),\n  )\n}\n A wrapper for an encoder and a decoder to convert from request/response data to and from a Gleam type\nBodyConverter(\n  encoder: fn(body_type) -&gt; string_builder.StringBuilder,\n  decoder: fn(String) -&gt;\n    Result(body_type, gleatter.GleatterError),\n)\n\n\nSynonyms:\ngleatter/body.BodyConverter\ngleatter/body BodyConverter","url":"gleatter/body.html#BodyConverter"},{"doc":"gleatter/body","title":"BodyType","content":"pub type BodyType {\n  EmptyBody\n  StringBody\n  JsonBody\n}\n The type of body that can be expected from a Route\nEmptyBody\n\n\nStringBody\n\n\nJsonBody\n\n\nSynonyms:\ngleatter/body.BodyType\ngleatter/body BodyType","url":"gleatter/body.html#BodyType"},{"doc":"gleatter/body","title":"RouteBody","content":"pub opaque type RouteBody(body_type)\n The body type of a Route\n\nSynonyms:\ngleatter/body.RouteBody\ngleatter/body RouteBody","url":"gleatter/body.html#RouteBody"},{"doc":"gleatter/body","title":"decode","content":"pub fn decode(\n  body: RouteBody(a),\n  value: String,\n) -&gt; Result(a, GleatterError)\n Decode a value using the RouteBody&#39;s decoder from a String\nSynonyms:\ngleatter/body.decode\ngleatter/body decode","url":"gleatter/body.html#decode"},{"doc":"gleatter/body","title":"empty_body","content":"pub fn empty_body() -&gt; RouteBody(Nil)\n Create a RouteBody that will be empty\nSynonyms:\ngleatter/body.empty_body\ngleatter/body empty_body","url":"gleatter/body.html#empty_body"},{"doc":"gleatter/body","title":"encode","content":"pub fn encode(body: RouteBody(a), value: a) -&gt; StringTree\n Encode a value using the RouteBody&#39;s encoder into a StringBuilder\nSynonyms:\ngleatter/body.encode\ngleatter/body encode","url":"gleatter/body.html#encode"},{"doc":"gleatter/body","title":"get_type","content":"pub fn get_type(body: RouteBody(a)) -&gt; BodyType\n Return the BodyType of a RouteBody\nSynonyms:\ngleatter/body.get_type\ngleatter/body get_type","url":"gleatter/body.html#get_type"},{"doc":"gleatter/body","title":"json_body","content":"pub fn json_body(\n  encoder: fn(a) -&gt; Json,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; RouteBody(a)\n Create a RouteBody that will be converted from/to json\nSynonyms:\ngleatter/body.json_body\ngleatter/body json_body","url":"gleatter/body.html#json_body"},{"doc":"gleatter/body","title":"string_body","content":"pub fn string_body(converter: BodyConverter(a)) -&gt; RouteBody(a)\n Create a RouteBody that will be converted from/to a string\nSynonyms:\ngleatter/body.string_body\ngleatter/body string_body","url":"gleatter/body.html#string_body"},{"doc":"gleatter/body","title":"gleatter/body","content":" This module exports types and functions related to the body of Routes","url":"gleatter/body.html"},{"doc":"gleatter/path","title":"PathConverter","content":"pub type PathConverter(path_type) {\n  PathConverter(\n    encoder: fn(path_type) -&gt; List(String),\n    decoder: fn(List(String)) -&gt; Result(path_type, Nil),\n  )\n}\n A wrapper for an encoder and a decoder to convert from request/response path to and from a Gleam type  \n The decoder should return Error(Nil) if the input path doesn&#39;t correspond to the expected pattern\nPathConverter(\n  encoder: fn(path_type) -&gt; List(String),\n  decoder: fn(List(String)) -&gt; Result(path_type, Nil),\n)\n\n\nSynonyms:\ngleatter/path.PathConverter\ngleatter/path PathConverter","url":"gleatter/path.html#PathConverter"},{"doc":"gleatter/path","title":"PathType","content":"pub type PathType {\n  StaticPath(root: List(String))\n  ComplexPath\n}\n The type of path that can be expected from a Route\nStaticPath(root: List(String))\n\n\nComplexPath\n\n\nSynonyms:\ngleatter/path.PathType\ngleatter/path PathType","url":"gleatter/path.html#PathType"},{"doc":"gleatter/path","title":"RoutePath","content":"pub opaque type RoutePath(path_type)\n The path type of a Route\n\nSynonyms:\ngleatter/path.RoutePath\ngleatter/path RoutePath","url":"gleatter/path.html#RoutePath"},{"doc":"gleatter/path","title":"complex_path","content":"pub fn complex_path(converter: PathConverter(a)) -&gt; RoutePath(a)\n Create a more complex RoutePath from a custom converter\nSynonyms:\ngleatter/path.complex_path\ngleatter/path complex_path","url":"gleatter/path.html#complex_path"},{"doc":"gleatter/path","title":"decode","content":"pub fn decode(\n  path: RoutePath(a),\n  value: List(String),\n) -&gt; Result(a, Nil)\n Decode a value using the RoutePath&#39;s decoder from path segments\nSynonyms:\ngleatter/path.decode\ngleatter/path decode","url":"gleatter/path.html#decode"},{"doc":"gleatter/path","title":"encode","content":"pub fn encode(path: RoutePath(a), value: a) -&gt; List(String)\n Encode a value using the RoutePath&#39;s encoder into path segments\nSynonyms:\ngleatter/path.encode\ngleatter/path encode","url":"gleatter/path.html#encode"},{"doc":"gleatter/path","title":"get_type","content":"pub fn get_type(path: RoutePath(a)) -&gt; PathType\n Return the PathType of a RoutePath\nSynonyms:\ngleatter/path.get_type\ngleatter/path get_type","url":"gleatter/path.html#get_type"},{"doc":"gleatter/path","title":"id_path","content":"pub fn id_path(root: List(String)) -&gt; RoutePath(String)\n Create a RoutePath that will match path of the type `/x/y/z/:id`\n The id has to be the last segment of the path\nSynonyms:\ngleatter/path.id_path\ngleatter/path id_path","url":"gleatter/path.html#id_path"},{"doc":"gleatter/path","title":"static_path","content":"pub fn static_path(root: List(String)) -&gt; RoutePath(Nil)\n Create a RoutePath that will solely match a static path\nSynonyms:\ngleatter/path.static_path\ngleatter/path static_path","url":"gleatter/path.html#static_path"},{"doc":"gleatter/path","title":"gleatter/path","content":" This module exports types and functions related to the path of Routes","url":"gleatter/path.html"},{"doc":"gleatter/query","title":"QueryConverter","content":"pub type QueryConverter(query_type) {\n  QueryConverter(\n    encoder: fn(query_type) -&gt; List(#(String, String)),\n    decoder: fn(List(#(String, String))) -&gt;\n      Result(query_type, Nil),\n  )\n}\n A wrapper for an encoder and a decoder to convert from request/response query to and from a Gleam type\nQueryConverter(\n  encoder: fn(query_type) -&gt; List(#(String, String)),\n  decoder: fn(List(#(String, String))) -&gt; Result(query_type, Nil),\n)\n\n\nSynonyms:\ngleatter/query.QueryConverter\ngleatter/query QueryConverter","url":"gleatter/query.html#QueryConverter"},{"doc":"gleatter/query","title":"QueryType","content":"pub type QueryType {\n  EmptyQuery\n  ComplexQuery\n}\n The type of query that can be expected from a Route\nEmptyQuery\n\n\nComplexQuery\n\n\nSynonyms:\ngleatter/query.QueryType\ngleatter/query QueryType","url":"gleatter/query.html#QueryType"},{"doc":"gleatter/query","title":"RouteQuery","content":"pub opaque type RouteQuery(query_type)\n The query type of a Route\n\nSynonyms:\ngleatter/query.RouteQuery\ngleatter/query RouteQuery","url":"gleatter/query.html#RouteQuery"},{"doc":"gleatter/query","title":"complex_query","content":"pub fn complex_query(\n  converter: QueryConverter(a),\n) -&gt; RouteQuery(a)\n Create a more complex RouteQuery from a custom converter\nSynonyms:\ngleatter/query.complex_query\ngleatter/query complex_query","url":"gleatter/query.html#complex_query"},{"doc":"gleatter/query","title":"decode","content":"pub fn decode(\n  query: RouteQuery(a),\n  value: List(#(String, String)),\n) -&gt; Result(a, Nil)\n Decode a value using the RouteQuery&#39;s decoder from a query\nSynonyms:\ngleatter/query.decode\ngleatter/query decode","url":"gleatter/query.html#decode"},{"doc":"gleatter/query","title":"empty_query","content":"pub fn empty_query() -&gt; RouteQuery(Nil)\n Create a RouteQuery that convert from/to an empty query\nSynonyms:\ngleatter/query.empty_query\ngleatter/query empty_query","url":"gleatter/query.html#empty_query"},{"doc":"gleatter/query","title":"encode","content":"pub fn encode(\n  query: RouteQuery(a),\n  value: a,\n) -&gt; List(#(String, String))\n Encode a value using the RouteQuery&#39;s encoder into a query\nSynonyms:\ngleatter/query.encode\ngleatter/query encode","url":"gleatter/query.html#encode"},{"doc":"gleatter/query","title":"get_type","content":"pub fn get_type(query: RouteQuery(a)) -&gt; QueryType\n Return the QueryType of a RouteQuery\nSynonyms:\ngleatter/query.get_type\ngleatter/query get_type","url":"gleatter/query.html#get_type"},{"doc":"gleatter/query","title":"gleatter/query","content":" This module exports types and functions related to the query of Routes","url":"gleatter/query.html"},{"doc":"gleatter/route","title":"Route","content":"pub type Route(\n  path_type,\n  query_type,\n  req_body_type,\n  res_body_type,\n) {\n  Route(\n    method: http.Method,\n    path: path.RoutePath(path_type),\n    query: query.RouteQuery(query_type),\n    req_body: body.RouteBody(req_body_type),\n    res_body: body.RouteBody(res_body_type),\n  )\n}\n The Route type  \n Routes should be built using the builder functions\nRoute(\n  method: http.Method,\n  path: path.RoutePath(path_type),\n  query: query.RouteQuery(query_type),\n  req_body: body.RouteBody(req_body_type),\n  res_body: body.RouteBody(res_body_type),\n)\n\n\nSynonyms:\ngleatter/route.Route\ngleatter/route Route","url":"gleatter/route.html#Route"},{"doc":"gleatter/route","title":"new","content":"pub fn new() -&gt; Route(Nil, Nil, Nil, Nil)\n Create a new Route  \n By default the method is GET and the path, query and bodies are empty\nSynonyms:\ngleatter/route.new\ngleatter/route new","url":"gleatter/route.html#new"},{"doc":"gleatter/route","title":"with_method","content":"pub fn with_method(\n  route: Route(a, b, c, d),\n  method: Method,\n) -&gt; Route(a, b, c, d)\n Change the method of a Route\nSynonyms:\ngleatter/route.with_method\ngleatter/route with_method","url":"gleatter/route.html#with_method"},{"doc":"gleatter/route","title":"with_path","content":"pub fn with_path(\n  route: Route(a, b, c, d),\n  path: RoutePath(e),\n) -&gt; Route(e, b, c, d)\n Change the path of a Route\nSynonyms:\ngleatter/route.with_path\ngleatter/route with_path","url":"gleatter/route.html#with_path"},{"doc":"gleatter/route","title":"with_query","content":"pub fn with_query(\n  route: Route(a, b, c, d),\n  query: RouteQuery(e),\n) -&gt; Route(a, e, c, d)\n Change the query of a Route\nSynonyms:\ngleatter/route.with_query\ngleatter/route with_query","url":"gleatter/route.html#with_query"},{"doc":"gleatter/route","title":"with_request_body","content":"pub fn with_request_body(\n  route: Route(a, b, c, d),\n  req_body: RouteBody(e),\n) -&gt; Route(a, b, e, d)\n Change the request body of a Route\nSynonyms:\ngleatter/route.with_request_body\ngleatter/route with_request_body","url":"gleatter/route.html#with_request_body"},{"doc":"gleatter/route","title":"with_response_body","content":"pub fn with_response_body(\n  route: Route(a, b, c, d),\n  res_body: RouteBody(e),\n) -&gt; Route(a, b, c, e)\n Change the response body of a Route\nSynonyms:\ngleatter/route.with_response_body\ngleatter/route with_response_body","url":"gleatter/route.html#with_response_body"},{"doc":"gleatter/route","title":"gleatter/route","content":" This module exports the Route type and builder functions.  \n Routes correspond to endpoints on a backend that will be called from a frontend.\n They are defined by a method, a path, a query and request/response bodies.","url":"gleatter/route.html"},{"doc":"gleatter/service","title":"RouteService","content":"pub type RouteService(base_type, upsert_type) {\n  RouteService(\n    root_path: List(String),\n    base: #(\n      fn(base_type) -&gt; json.Json,\n      fn(dynamic.Dynamic) -&gt;\n        Result(base_type, List(dynamic.DecodeError)),\n    ),\n    upsert: #(\n      fn(upsert_type) -&gt; json.Json,\n      fn(dynamic.Dynamic) -&gt;\n        Result(upsert_type, List(dynamic.DecodeError)),\n    ),\n  )\n}\n The RouteService type  \n Contains the data necessary to build the CRUD routes.\n Note that all data transmission will be done via JSON objects.\nRouteService(\n  root_path: List(String),\n  base: #(\n    fn(base_type) -&gt; json.Json,\n    fn(dynamic.Dynamic) -&gt;\n      Result(base_type, List(dynamic.DecodeError)),\n  ),\n  upsert: #(\n    fn(upsert_type) -&gt; json.Json,\n    fn(dynamic.Dynamic) -&gt;\n      Result(upsert_type, List(dynamic.DecodeError)),\n  ),\n)\n\n\nSynonyms:\ngleatter/service.RouteService\ngleatter/service RouteService","url":"gleatter/service.html#RouteService"},{"doc":"gleatter/service","title":"create_route","content":"pub fn create_route(\n  service: RouteService(a, b),\n) -&gt; Route(Nil, Nil, b, a)\n Generate a create route associated with a service\nSynonyms:\ngleatter/service.create_route\ngleatter/service create_route","url":"gleatter/service.html#create_route"},{"doc":"gleatter/service","title":"delete_route","content":"pub fn delete_route(\n  service: RouteService(a, b),\n) -&gt; Route(String, Nil, Nil, String)\n Generate a delete route associated with a service  \n Note that the return is the id of the deleted instance\nSynonyms:\ngleatter/service.delete_route\ngleatter/service delete_route","url":"gleatter/service.html#delete_route"},{"doc":"gleatter/service","title":"get_all_route","content":"pub fn get_all_route(\n  service: RouteService(a, b),\n) -&gt; Route(Nil, Nil, Nil, List(a))\n Generate a get-all route associated with a service\nSynonyms:\ngleatter/service.get_all_route\ngleatter/service get_all_route","url":"gleatter/service.html#get_all_route"},{"doc":"gleatter/service","title":"get_route","content":"pub fn get_route(\n  service: RouteService(a, b),\n) -&gt; Route(String, Nil, Nil, a)\n Generate a get route associated with a service\nSynonyms:\ngleatter/service.get_route\ngleatter/service get_route","url":"gleatter/service.html#get_route"},{"doc":"gleatter/service","title":"new","content":"pub fn new() -&gt; RouteService(Nil, Nil)\n Create a new empty service  \n The base and upsert types will have to be specified !\nSynonyms:\ngleatter/service.new\ngleatter/service new","url":"gleatter/service.html#new"},{"doc":"gleatter/service","title":"update_route","content":"pub fn update_route(\n  service: RouteService(a, b),\n) -&gt; Route(String, Nil, b, a)\n Generate a update route associated with a service\nSynonyms:\ngleatter/service.update_route\ngleatter/service update_route","url":"gleatter/service.html#update_route"},{"doc":"gleatter/service","title":"with_base_converter","content":"pub fn with_base_converter(\n  service: RouteService(a, b),\n  converter: Converter(c),\n) -&gt; RouteService(c, b)\n Specify the base type of a service by providing a gleatter_convert type  \n The base type of a service represent the type of object your service is associated with\nSynonyms:\ngleatter/service.with_base_converter\ngleatter/service with_base_converter","url":"gleatter/service.html#with_base_converter"},{"doc":"gleatter/service","title":"with_base_type","content":"pub fn with_base_type(\n  service: RouteService(a, b),\n  base_encoder: fn(c) -&gt; Json,\n  base_decoder: fn(Dynamic) -&gt; Result(c, List(DecodeError)),\n) -&gt; RouteService(c, b)\n Specify the base type of a service by providing a JSON encoder &amp; decoder  \n The base type of a service represent the type of object your service is associated with\nSynonyms:\ngleatter/service.with_base_type\ngleatter/service with_base_type","url":"gleatter/service.html#with_base_type"},{"doc":"gleatter/service","title":"with_root_path","content":"pub fn with_root_path(\n  service: RouteService(a, b),\n  root_path: List(String),\n) -&gt; RouteService(a, b)\n Change the root path of a service\nSynonyms:\ngleatter/service.with_root_path\ngleatter/service with_root_path","url":"gleatter/service.html#with_root_path"},{"doc":"gleatter/service","title":"with_upsert_converter","content":"pub fn with_upsert_converter(\n  service: RouteService(a, b),\n  converter: Converter(c),\n) -&gt; RouteService(a, c)\n Specify the upsert type of a service by providing a gleatter_convert type    \n The upsert type of a service represent the type used to create or update objects of your service\nSynonyms:\ngleatter/service.with_upsert_converter\ngleatter/service with_upsert_converter","url":"gleatter/service.html#with_upsert_converter"},{"doc":"gleatter/service","title":"with_upsert_type","content":"pub fn with_upsert_type(\n  service: RouteService(a, b),\n  upsert_encoder: fn(c) -&gt; Json,\n  upsert_decoder: fn(Dynamic) -&gt; Result(c, List(DecodeError)),\n) -&gt; RouteService(a, c)\n Specify the upsert type of a service by providing a JSON encoder &amp; decoder  \n The upsert type of a service represent the type used to create or update objects of your service\nSynonyms:\ngleatter/service.with_upsert_type\ngleatter/service with_upsert_type","url":"gleatter/service.html#with_upsert_type"},{"doc":"gleatter/service","title":"gleatter/service","content":" This module helps you create standard CRUD services routes\n \n Example : \n ```\n import gleatter/service\n import convert\n \n pub type Todo {\n  Todo(id: String, title: String)\n }\n \n pub type UpsertTodo {\n  UpsertTodo(title: String)\n }\n \n pub fn todo_converter() -&gt; convert.Converter(Todo) {\n  convert.object({\n    use id &lt;- convert.parameter\n    use title &lt;- convert.parameter\n    use &lt;- convert.constructor\n    Todo(id:, title:)\n  })\n  |&gt; convert.field(&quot;id&quot;, fn(v) { v.id }, convert.string())\n  |&gt; convert.field(&quot;title&quot;, fn(v) { v.title }, convert.string())\n  |&gt; convert.to_converter()\n }\n \n pub fn upsert_todo_converter() -&gt; convert.Converter(UpsertTodo) {\n  convert.object({\n    use title &lt;- convert.parameter\n    use &lt;- convert.constructor\n    UpsertTodo(title:)\n  })\n  |&gt; convert.field(&quot;title&quot;, fn(v) { v.title }, convert.string())\n  |&gt; convert.to_converter()\n }\n \n pub fn todo_service() -&gt; service.RouteService(Todo, UpsertTodo) {\n  service.new()\n  |&gt; service.with_root_path([&quot;todos&quot;])\n  |&gt; service.with_base_converter(todo_converter())\n  |&gt; service.with_upsert_converter(upsert_todo_converter())\n }\n \n pub fn create_todo_route() {\n  todo_service() |&gt; service.create_route()\n }\n ```","url":"gleatter/service.html"},{"doc":"gleatter/utils","title":"to_request","content":"pub fn to_request(\n  route: Route(a, b, c, d),\n  path: a,\n  query: b,\n  body: c,\n) -&gt; Request(String)\n Converts a Route to a HTTP request by providing the path, query and request body data.  \n This function is actually not used in the library, it stays there in case one needs it.\nSynonyms:\ngleatter/utils.to_request\ngleatter/utils to_request","url":"gleatter/utils.html#to_request"},{"doc":"gleatter/utils","title":"gleatter/utils","content":"","url":"gleatter/utils.html"}]);